@@ -8,13 +8,14 @@ try:
     import sys
     import zipfile
     import shutil
-    import cgi
+    import html
     import subprocess
     import threading
     import time
     import dbus
     from PIL import Image
     import datetime
+    import proxygsettings
 except Exception as detail:
     print(detail)
     sys.exit(1)
@@ -385,7 +386,12 @@ class Spice_Harvester(GObject.Object):
         parsed_url = urlparse(url)
         host = parsed_url.netloc
         try:
-            connection = HTTPSConnection(host, timeout=15)
+            proxy = proxygsettings.get_proxy_settings()
+            if proxy and proxy.get('https'):
+                connection = HTTPSConnection(proxy.get('https'), timeout=15)
+                connection.set_tunnel(host)
+            else:
+                connection = HTTPSConnection(host, timeout=15)
             headers = { "Accept-Encoding": "identity", "Host": host, "User-Agent": "Python/3" }
             connection.request("GET", parsed_url.path, headers=headers)
             urlobj = connection.getresponse()
@@ -446,7 +452,7 @@ class Spice_Harvester(GObject.Object):
             return False
 
         try:
-            return self.meta_map[uuid]["last-edited"] < self.index_cache[uuid]["last_edited"]
+            return int(self.meta_map[uuid]["last-edited"]) < self.index_cache[uuid]["last_edited"]
         except Exception as e:
             return False
 
@@ -726,7 +732,7 @@ class Spice_Harvester(GObject.Object):
         markup = msg
         if detail is not None:
             markup += _("\n\nDetails:  %s") % (str(detail))
-        esc = cgi.escape(markup)
+        esc = html.escape(markup)
         dialog.set_markup(esc)
         dialog.show_all()
         response = dialog.run()
@@ -774,7 +780,14 @@ class Spice_Harvester(GObject.Object):
         enabled_extensions = self.settings.get_strv(self.enabled_key)
         new_list = []
         for enabled_extension in enabled_extensions:
-            if uuid not in enabled_extension:
+            if self.collection_type == 'applet':
+                enabled_uuid = enabled_extension.split(':')[3].strip('!')
+            elif self.collection_type == 'desklet':
+                enabled_uuid = enabled_extension.split(':')[0].strip('!')
+            else:
+                enabled_uuid = enabled_extension
+
+            if enabled_uuid != uuid:
                 new_list.append(enabled_extension)
         self.settings.set_strv(self.enabled_key, new_list)
 
