@@ -1,5 +1,6 @@
 #!/usr/bin/python3
 
+import getopt
 import sys
 
 import os
@@ -12,6 +13,7 @@ import urllib.request as urllib
 from functools import cmp_to_key
 import unicodedata
 import config
+from setproctitle import setproctitle
 
 import gi
 gi.require_version('Gtk', '3.0')
@@ -25,12 +27,12 @@ import proxygsettings
 import SettingsWidgets
 
 # i18n
-gettext.install("cinnamon", "/usr/share/locale", names="ngettext")
+gettext.install("cinnamon", "/usr/share/locale", names=["ngettext"])
 
 # Standard setting pages... this can be expanded to include applet dirs maybe?
 mod_files = glob.glob(config.currentPath + "/modules/*.py")
 mod_files.sort()
-if len(mod_files) is 0:
+if len(mod_files) == 0:
     print("No settings modules found!!")
     sys.exit(1)
 
@@ -79,7 +81,7 @@ STANDALONE_MODULES = [
     [_("Firewall"),                      "gufw",                                "cs-firewall",        "admin",          _("firewall, block, filter, programs")],
     [_("Firewall"),                      "firewall-config",                     "cs-firewall",        "admin",          _("firewall, block, filter, programs")],
     [_("Languages"),                     "mintlocale",                          "cs-language",        "prefs",          _("language, install, foreign")],
-    [_("Input Method"),                  "mintlocale im",                       "cs-input-method",    "prefs",          _("language, install, foreign, input, method, chinese, korean, japanese, typing")],
+    [_("Input Method"),                  "mintlocale-im",                       "cs-input-method",    "prefs",          _("language, install, foreign, input, method, chinese, korean, japanese, typing")],
     [_("Login Window"),                  "pkexec lightdm-settings",             "cs-login",           "admin",          _("login, lightdm, mdm, gdm, manager, user, password, startup, switch")],
     [_("Login Window"),                  "lightdm-gtk-greeter-settings-pkexec", "cs-login",           "admin",          _("login, lightdm, manager, settings, editor")],
     [_("Driver Manager"),                "pkexec driver-manager",               "cs-drivers",         "admin",          _("video, driver, wifi, card, hardware, proprietary, nvidia, radeon, nouveau, fglrx")],
@@ -92,6 +94,56 @@ STANDALONE_MODULES = [
     [_("Manage Services and Units"),     "systemd-manager-pkexec",              "cs-sources",         "admin",          _("systemd, units, services, systemctl, init")]
 ]
 
+TABS = {
+    # KEY (cs_KEY.py) : {"tab_name": tab_number, ... }
+    "universal-access": {"visual": 0, "keyboard": 1, "typing": 2, "mouse": 3},
+    "applets":          {"installed": 0, "more": 1, "download": 1},
+    "backgrounds":      {"images": 0, "settings": 1},
+    "default":          {"preferred": 0, "removable": 1},
+    "desklets":         {"installed": 0, "more": 1, "download": 1, "general": 2},
+    "effects":          {"effects": 0, "customize": 1},
+    "extensions":       {"installed": 0, "more": 1, "download": 1},
+    "keyboard":         {"typing": 0, "shortcuts": 1, "layouts": 2},
+    "mouse":            {"mouse": 0, "touchpad": 1},
+    "power":            {"power": 0, "batteries": 1, "brightness": 2},
+    "screensaver":      {"settings": 0, "customize": 1},
+    "sound":            {"output": 0, "input": 1, "sounds": 2, "applications": 3, "settings": 4},
+    "themes":           {"themes": 0, "download": 1, "options": 2},
+    "windows":          {"titlebar": 0, "behavior": 1, "alttab": 2},
+    "workspaces":       {"osd": 0, "settings": 1}
+}
+
+ARG_REWRITE = {
+    'accessibility':    'universal-access',
+    'screen':           'display',
+    'screens':          'display',
+    'bluetooth':        'blueberry',
+    'hotcorners':       'hotcorner',
+    'accounts':         'online-accounts',
+    'colors':           'color',
+    'me':               'user',
+    'lightdm-settings': 'pkexec lightdm-settings',
+    'login-screen':     'pkexec lightdm-settings',
+    'window':           'windows',
+    'background':       'backgrounds',
+    'driver-manager':   'pkexec driver-manager',
+    'drivers':          'pkexec driver-manager',
+    'printers':         'system-config-printer',
+    'printer':          'system-config-printer',
+    'infos':            'info',
+    'locale':           'mintlocale',
+    'language':         'mintlocale',
+    'input-method':     'mintlocale-im',
+    'nvidia':           'nvidia-settings',
+    'firewall':         'gufw',
+    'networks':         'network',
+    'sources':          'pkexec mintsources',
+    'mintsources':      'pkexec mintsources',
+    'panels':           'panel',
+    'tablet':           'wacom',
+    'users':            'cinnamon-settings-users'
+}
+
 def print_timing(func):
     # decorate functions with @print_timing to output how long they take to run.
     def wrapper(*arg):
@@ -115,7 +167,7 @@ class MainWindow:
             self.deselect(cat)
             filtered_path = side_view.get_model().convert_path_to_child_path(selected_items[0])
             if filtered_path is not None:
-                self.go_to_sidepage(cat, filtered_path, animate=True)
+                self.go_to_sidepage(cat, filtered_path, user_action=True)
 
     def _on_sidepage_hide_stack(self):
         self.stack_switcher.set_opacity(0)
@@ -123,19 +175,29 @@ class MainWindow:
     def _on_sidepage_show_stack(self):
         self.stack_switcher.set_opacity(1)
 
-    def go_to_sidepage(self, cat, path, animate=True):
+    def go_to_sidepage(self, cat, path, user_action=True):
         iterator = self.store[cat].get_iter(path)
         sidePage = self.store[cat].get_value(iterator,2)
         if not sidePage.is_standalone:
-            self.window.set_title(sidePage.name)
-            self.window.set_icon_name(sidePage.icon)
+            if not user_action:
+                self.window.set_title(sidePage.name)
+                self.window.set_icon_name(sidePage.icon)
             sidePage.build()
             if sidePage.stack:
                 current_page = sidePage.stack.get_visible_child_name()
                 self.stack_switcher.set_stack(sidePage.stack)
                 l = sidePage.stack.get_children()
                 if len(l) > 0:
-                    sidePage.stack.set_visible_child(l[0])
+                    if self.tab in range(len(l)):
+                        sidePage.stack.set_visible_child(l[self.tab])
+                        visible_child = sidePage.stack.get_visible_child()
+                        if self.tab == 1 \
+                        and hasattr(visible_child, 'sort_combo') \
+                        and self.sort in range(4):
+                            visible_child.sort_combo.set_active(self.sort)
+                            visible_child.sort_changed()
+                    else:
+                        sidePage.stack.set_visible_child(l[0])
                     if sidePage.stack.get_visible():
                         self.stack_switcher.set_opacity(1)
                     else:
@@ -147,7 +209,7 @@ class MainWindow:
                     self.stack_switcher.set_opacity(0)
             else:
                 self.stack_switcher.set_opacity(0)
-            if animate:
+            if user_action:
                 self.main_stack.set_visible_child_name("content_box_page")
                 self.header_stack.set_visible_child_name("content_box")
 
@@ -195,6 +257,7 @@ class MainWindow:
         self.builder.add_from_file(config.currentPath + "/cinnamon-settings.ui")
         self.window = XApp.GtkWindow(window_position=Gtk.WindowPosition.CENTER,
                                      default_width=800, default_height=600)
+
         main_box = self.builder.get_object("main_box")
         self.window.add(main_box)
         self.top_bar = self.builder.get_object("top_bar")
@@ -221,18 +284,12 @@ class MainWindow:
 
         self.stack_switcher = self.builder.get_object("stack_switcher")
 
-        m, n = self.button_back.get_preferred_width()
-        self.stack_switcher.set_margin_end(n)
-
         self.search_entry = self.builder.get_object("search_box")
         self.search_entry.set_placeholder_text(_("Search"))
         self.search_entry.connect("changed", self.onSearchTextChanged)
         self.search_entry.connect("icon-press", self.onClearSearchBox)
 
         self.window.connect("destroy", self.quit)
-        self.window.connect("key-press-event", self.on_keypress)
-        self.window.connect("button-press-event", self.on_buttonpress)
-        self.window.show()
 
         self.builder.connect_signals(self)
         self.unsortedSidePages = []
@@ -316,16 +373,74 @@ class MainWindow:
 
         self.calculate_bar_heights()
 
+        self.tab = 0 # open 'manage' tab by default
+        self.sort = 1 # sorted by 'score' by default
+
         # Select the first sidePage
-        if len(sys.argv) > 1 and sys.argv[1] in sidePagesIters:
-            (iter, cat) = sidePagesIters[sys.argv[1]]
+        if len(sys.argv) > 1:
+            arg1 = sys.argv[1]
+            if arg1 in ARG_REWRITE.keys():
+                arg1 = ARG_REWRITE[arg1]
+        if len(sys.argv) > 1 and arg1 in sidePagesIters:
+            # Analyses arguments to know the tab to open
+            # and the sort to apply if the tab is the 'more' one.
+            # Examples:
+            #   cinnamon-settings.py applets --tab=more --sort=date
+            #   cinnamon-settings.py applets --tab=1 --sort=2
+            #   cinnamon-settings.py applets --tab=more --sort=date
+            #   cinnamon-settings.py applets --tab=1 -s 2
+            #   cinnamon-settings.py applets -t 1 -s installed
+            #   cinnamon-settings.py desklets -t 2
+            # Please note that useless or wrong arguments are ignored.
+            opts = []
+            sorts_literal = {"name":0, "score":1, "date":2, "installed":3}
+            tabs_literal = {"default":0}
+            if arg1 in TABS.keys():
+                tabs_literal = TABS[arg1]
+
+            try:
+                if len(sys.argv) > 2:
+                    opts = getopt.getopt(sys.argv[2:], "t:s:", ["tab=", "sort="])[0]
+            except getopt.GetoptError:
+                pass
+
+            for opt, arg in opts:
+                if opt in ("-t", "--tab"):
+                    if arg.isdecimal():
+                        self.tab = int(arg)
+                    elif arg in tabs_literal.keys():
+                        self.tab = tabs_literal[arg]
+                if opt in ("-s", "--sort"):
+                    if arg.isdecimal():
+                        self.sort = int(arg)
+                    elif arg in sorts_literal.keys():
+                        self.sort = sorts_literal[arg]
+
+            # If we're launching a module directly, set the WM class so GWL
+            # can consider it as a standalone app and give it its own
+            # group.
+            wm_class = "cinnamon-settings %s" % arg1
+            self.window.set_wmclass(wm_class, wm_class)
+            self.button_back.hide()
+            (iter, cat) = sidePagesIters[arg1]
             path = self.store[cat].get_path(iter)
             if path:
-                self.go_to_sidepage(cat, path, animate=False)
+                self.go_to_sidepage(cat, path, user_action=False)
+                self.window.show()
+                if arg1 in ("mintlocale", "blueberry", "system-config-printer", \
+                            "mintlocale-im", "nvidia-settings"):
+                    # These modules do not need to leave the System Settings window open,
+                    # when selected by command line argument.
+                    self.window.close()
             else:
                 self.search_entry.grab_focus()
+                self.window.show()
         else:
             self.search_entry.grab_focus()
+            self.window.connect("key-press-event", self.on_keypress)
+            self.window.connect("button-press-event", self.on_buttonpress)
+
+            self.window.show()
 
     def on_keypress(self, widget, event):
         grab = False
@@ -358,15 +473,8 @@ class MainWindow:
             self.search_entry.set_text("")
 
     def strip_accents(self, text):
-        try:
-            text = unicode(text, 'utf-8')
-        except NameError:
-            # unicode is default in Python 3
-            pass
-        text = unicodedata.normalize('NFD', text)
-        text = text.encode('ascii', 'ignore')
-        text = text.decode("utf-8")
-        return str(text)
+        text = unicodedata.normalize('NFKD', text)
+        return ''.join([c for c in text if not unicodedata.combining(c)])
 
     def filter_visible_function(self, model, iter, user_data = None):
         sidePage = model.get_value(iter, 2)
@@ -469,16 +577,21 @@ class MainWindow:
             ivw = iconview.get_window()
             iv_x, iv_y = ivw.get_position()
 
-            final_y = rect.y + (rect.height / 2) + cw_y + iv_y
+            final_y = rect.y + cw_y + iv_y
 
             adj = self.side_view_sw.get_vadjustment()
             page = adj.get_page_size()
             current_pos = adj.get_value()
 
-            if final_y > current_pos + page:
-                adj.set_value(iv_y + rect.y)
-            elif final_y < current_pos:
-                adj.set_value(iv_y + rect.y)
+            if (final_y > 0) and ((final_y + rect.height) < page):
+                return
+
+            if ((final_y + rect.height) > page):
+                adj.set_value(current_pos + final_y + rect.height - page + 10)
+            elif final_y < 0:
+                # We can just add a negative here (since final_y < 0), but it's less
+                # confusing to be explicit that we're decreasing current_pos.
+                adj.set_value(current_pos - abs(final_y) - 10)
 
     def on_selection_changed(self, widget, category):
         sel = widget.get_selected_items()
@@ -598,6 +711,7 @@ class MainWindow:
         Gtk.main_quit()
 
 if __name__ == "__main__":
+    setproctitle("cinnamon-settings")
     import signal
 
     ps = proxygsettings.get_proxy_settings()
